#!/usr/bin/python 
# Itch.py
# Some simple, mostly Mac-specific, functionality for X-Chat
# (cc) 2006 Filipp Lepalaan <filipp@mac.com>

# http://labix.org/xchat-python
# http://docs.python.org/lib/string-methods.html#string-methods
# http://docs.python.org/ref/

# TODO:
# - Add some simple /away management
# - Add some last.fm stuff http://www.last.fm/tools/feeds/
# - iDisk uploads
# - XDCC bots into separate window
# - macupdate.com
# - temps (Mac & outside)
# - profile support (from AB)

__module_name__ = 'Itch'
__module_version__ = '0.8'
__module_description__ = 'Simple announcement and utility script for OS X. (cc) Filipp Lepalaan <filipp@mac.com>'

import xchat
import commands
import re

from math import floor
from time import time, gmtime, strftime
from datetime import datetime
from string import capwords, lower

# Some basic variables
away_start = 0
myNick = xchat.get_info ('nick')
changeNick = False
chanLangs = {'emug' : 'et', 'script-test' : 'en'}	# Could use this to localize command output
awayHook = None

# function-specific global variables
spOpts = ' | '.join (['hardware','software','network','power','displays'])
hwOpts = ' | '.join (['temps', 'mem', 'proc', 'hd'])

# Simple RSS support
# def getRSS:
# http://www.macupdate.com/mommy/macsurferx.xml
# Define callbacks
def nowPlaying (word, word_eol, userdata):
	word.append ('eol')
	info = commands.getoutput ('/usr/bin/osascript Plugins/LittleHelper.scpt ' + word[1])
	if info == "false":
		return xchat.EAT_ALL
	if word[1] == "send":
		return xchat.command ('dcc send ' + word[2] + ' "' + info + '"')
	xchat.command ('me // ' + info)
	return xchat.EAT_ALL

# Away management
def way (word, word_eol, userdata):
	global away_start
	global awayHook
	
	if len (word) > 1:
		reason = word[1]
	else:
		reason = 'for no reason'
	
	xchat.command ('away')
	
	if xchat.get_info ('away') == None:
		away_start = datetime.today ()
		awayHook = xchat.hook_print ('Channel Msg Hilight', pageAway)
		xchat.command ('me has gone away (' + reason + ')')
		return xchat.EAT_ALL
	else:
		if awayHook != None:
			xchat.unhook (awayHook)
		td = datetime.today () - away_start
		minutes, seconds = divmod (td.seconds, 60)
		hours, minutes = divmod (minutes, 60)
		xchat.command ("me is back (gone %dh:%dm:%ds)" % (hours, minutes, seconds))
		return xchat.EAT_ALL

# Page away messages to a query window
# http://forum.xchat.org/viewtopic.php?t=1085&view=previous&sid=df22a893e5c8f2c6ba2d2f9a15eb3415
# http://docs.python.org/lib/datetime-date.html
def pageAway (word, word_eol, userdata): 
	#xchat.command ('query >>pager<<')
	# Still all awaymsgs into one query window:
	#xchat.command ('newserver -noconnect >>away<<')
	ac = xchat.find_context (channel='>>pager<<')
	ac.prnt (strftime ('%H:%M') + ': ' + word_eol[0])
	return None

# Pedal to the metal
def getHardware (word, word_eol, userdata):
	global hwOpts
	if len (word) == 1:
		print 'Usage: ' + word[0] + hwOpts
	
	ioreg = commands.getoutput ('/usr/sbin/ioreg -n IOHWSensor')
	k = re.findall ("\"location\" = \"(.*?)\"", ioreg)
	v = re.findall ("\"current-value\" = (\d*)", ioreg)
	
	print dict (zip ([lower (el) for el in k], [int (el) >> 16 for el in v]))

	return xchat.EAT_ALL
	
def putFile (word, word_eol, userdata):
	return xchat.EAT_ALL

def macUpdate (word, word_eol, userdata):
	return xchat.EAT_ALL

def systemProfiler (word, word_eol, userdata):
	global spOpts
	if len (word) < 2:
		print 'Usage: /' + word[0] + ' ' + spOpts
		return xchat.EAT_ALL
	print commands.getoutput ('/usr/sbin/system_profiler SP' + capwords (word[1]) + 'DataType')
	return xchat.EAT_ALL

def setNote (word, word_eol, userdata):
	if len (word) < 2:
		print 'Usage: /' + word[0] + ' nick note'
		return xchat.EAT_ALL
	
	return xchat.EAT_ALL
	
def getCalendar (word, word_eol, userdata):
	print commands.getoutput ('grep -h `date +%m/%d` /usr/share/calendar/calendar.*')
	return xchat.EAT_ALL

# Thanks to pea!
def iSay (word, word_eol, userdata):
	out = ''
	if word[1] == 'r':
		xchat.command ('say ' + ' '.join (word [3:]) * int (word[2]))
		return xchat.EAT_ALL
	
	for w in word[1:]:
		out += 'i' + capwords (w.lstrip ('i')) + ' '
	
	xchat.command ('say ' + out)
	return xchat.EAT_ALL

def unload (userdata):
	print "> %s %s reloaded" % (__module_name__, __module_version__)
	
# Hook it up
xchat.hook_command ('np', nowPlaying, help = '/np [full | qt] [send nick] Show (or send) what\'s currently playing in iTunes or QT Player')
xchat.hook_command ('way', way, help = '/way [reason] Go away or come back')
xchat.hook_command ('mu', macUpdate, help = '/mu [top] [#] Show macupdate.com hot picks')
xchat.hook_command ('put', putFile, help = '/put <file> Upload something to putfile.com and return the URL')
xchat.hook_command ('sp', systemProfiler, help = '/sp ' + spOpts + ' Show System Profiler data')
xchat.hook_command ('isay', iSay, help = '/isay [r times] something iSay iSomething')
xchat.hook_command ('hw', getHardware, help = '/hw ' + hwOpts + ' show hardware info')
xchat.hook_command ('day', getCalendar, help = '/day print what happened today in history')
xchat.hook_command ('note', setNote, help = '/note nick note Send note to nick when they join')

xchat.hook_unload (unload)
